@@ -6,8 +6,7 @@ use std::process;
 use std::thread;
 
 use ipnetwork::NetworkSize;
-use pnet::datalink;
-use pnet::datalink::{MacAddr};
+use pnet::datalink::{self, MacAddr};
 use clap::{Arg, App};
 
 const FIVE_HOURS: u64 = 5 * 60 * 60; 
@@ -16,7 +15,7 @@ const TIMEOUT_DEFAULT: u64 = 2;
 fn main() {
 
     let matches = App::new("arp-scan")
-        .version("0.4.0")
+        .version("0.5.0")
         .about("A minimalistic ARP scan tool written in Rust")
         .arg(
             Arg::with_name("interface").short("i").long("interface").takes_value(true).value_name("INTERFACE_NAME").help("Network interface")
@@ -33,6 +32,9 @@ fn main() {
         .arg(
             Arg::with_name("numeric").short("n").long("numeric").takes_value(false).help("Numeric mode, no hostname resolution")
         )
+        .arg(
+            Arg::with_name("vlan").short("Q").long("vlan").takes_value(true).value_name("VLAN_ID").help("Send using 802.1Q with VLAN ID")
+        )
         .arg(
             Arg::with_name("list").short("l").long("list").takes_value(false).help("List network interfaces")
         )
@@ -58,11 +60,19 @@ fn main() {
     // with an IPv4 address and root permissions (for crafting ARP packets).
 
     let interface_name = match matches.value_of("interface") {
-        Some(name) => name,
+        Some(name) => String::from(name),
         None => {
-            eprintln!("Network interface name required");
-            eprintln!("Use 'arp scan -l' to list available interfaces");
-            process::exit(1);
+
+            match utils::select_default_interface() {
+                Some(default_interface) => {
+                    String::from(default_interface.name)
+                },
+                None => {
+                    eprintln!("Network interface name required");
+                    eprintln!("Use 'arp scan -l' to list available interfaces");
+                    process::exit(1);
+                }
+            }
         }
     };
 
@@ -104,6 +114,20 @@ fn main() {
         },
         None => None
     };
+
+    let vlan_id: Option<u16> = match matches.value_of("vlan") {
+        Some(vlan) => {
+
+            match vlan.parse::<u16>() {
+                Ok(vlan_number) => Some(vlan_number),
+                Err(_) => {
+                    eprintln!("Expected valid VLAN identifier");
+                    process::exit(1);
+                }
+            }
+        },
+        None => None
+    };
     
     if !utils::is_root_user() {
         eprintln!("Should run this binary as root");
@@ -162,7 +186,7 @@ fn main() {
     for ip_address in ip_network.iter() {
 
         if let IpAddr::V4(ipv4_address) = ip_address {
-            network::send_arp_request(&mut tx, selected_interface, ipv4_address, source_ipv4, destination_mac);
+            network::send_arp_request(&mut tx, selected_interface, ipv4_address, source_ipv4, destination_mac, vlan_id);
         }
     }
 
